package cz.zcu.kiv.aswi.maploc.locator;

import java.util.Map;
import java.util.Set;

/**
 * Interface pro lokatory
 */
public interface Locator {
	/**
	 * Vraci pozici objektu zadaneho pomoci parametru id. Pozice je zde reprezentovana polem o dvou prvcich: zemepisna sirka a delka
	 * @param id		id objektu
	 * @return			pole o dvou prvcich: sirka a delka
	 */
	double[] getLocation(long id);
	/**
	 * Vytvori novy objekt s parametry zadanymi pomoci parametru parameters.
	 * @param parameters	parametry pridaneho objektu
	 * @return				id pridaneho objektu
	 */
    long addObject(Map<String, Object> parameters);
    /**
     * Smaze zadany objekt pomoci jeho id.
     * @param id			id objektu
     * @return				true, pokud se povedlo
     */
    boolean deleteObject(long id);
    /**
     * Vraci seznam parametru, ktere lze pouzit pri zakladani objektu
     * @return
     */
    Set<String> getParamNames();
    /**
     * Vraci unikatni identifikator lokatoru
     * @return				unikatni identifikator lokatoru
     */
    String getIdentificator();
    /**
     * Vraci stav objektu a dalsi ruzne parametry, ktere se meni v prubehu existence objektu
     * @param id			id objektu
     * @return				parametry objektu
     */
    Map<String, Object> getObjectStatus(long id);
}