package cz.zcu.kiv.aswi.svk.lokatory.nmealocator;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class Loader extends Thread {

	private List<GeoPoint> locations;
	private int id;

	private DateFormat dateFormatParameters = new SimpleDateFormat("EEE, d. MMM yyyy HH:mm:ss");
	private DateFormat dateFormatNMEALong = new SimpleDateFormat("ddMMyyHHmmss.SSS");
	private DateFormat dateFormatNMEAShort = new SimpleDateFormat("ddMMyyHHmmss");

	private BufferedReader input;

	private long dateBegin = 0;
	private long dateEnd = Long.MAX_VALUE;


	public Loader(List<GeoPoint> locations, Map<String, Object> parameters, int id) {
		super();
		this.locations = locations;
		this.id = id;

		try {
			input = new BufferedReader(new FileReader(parameters.get("file").toString()));
		} catch (Exception e) {
			throw new IllegalArgumentException("Error with opening file.", e);
		}

		// nastaveni locale pro parsovani datumu
		Locale.setDefault(Locale.US);

		//nacteni parametru begin a end
		try {
			dateBegin = dateFormatParameters.parse(parameters.get("begin").toString()).getTime();
		} catch (Exception e) {
			//pouziji se vychozi hodnoty
		}
		try {
			dateEnd = dateFormatParameters.parse(parameters.get("end").toString()).getTime();
		} catch (Exception e) {
			//pouziji se vychozi hodnoty
		}
	}

	@Override
	public void run() {
		String line;
		String[] words;
		long lastTimeStamp = 0;

		try {
			while ((line = input.readLine()) != null) {
				if ("$GPRMC".equals(line.substring(0, 6))) {
					words = line.split(",");
					long timeStamp = parseTimeStamp(words[9] + words[1]);

					if ((timeStamp < dateBegin)) continue;
					if (timeStamp > dateEnd) break;

					double latitude = parseLocation(words[3], words[4].charAt(0));
					double longitude = parseLocation(words[5], words[6].charAt(0));

					if (timeStamp - lastTimeStamp >= 1000) {
						if (lastTimeStamp == 0)
							lastTimeStamp = timeStamp - 1000;
						saveLocation(timeStamp, latitude, longitude, timeStamp - lastTimeStamp);
						lastTimeStamp = locations.get(id).getTimeStamp();
					}
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
		}

		locations.set(id, null);
	}

	private double parseLocation(String location, char latLonChar) {
		double value = Double.parseDouble(location);
		int degree = (int) (value / 100);
		value-= degree * 100;

		if (Character.toUpperCase(latLonChar) == 'S' || Character.toUpperCase(latLonChar) == 'W')
			degree = -degree;

		return ((double) degree) + value / 60;
	}

	private long parseTimeStamp(String timeStamp) {
		try {
			return dateFormatNMEALong.parse(timeStamp).getTime();
		} catch (ParseException e) {
			try {
				return dateFormatNMEAShort.parse(timeStamp).getTime();
			} catch (ParseException e1) {
				return 0;
			}
		}
	}

	synchronized private void saveLocation(long timestamp, double latitude, double longitude, long waitTime) throws InterruptedException {
		locations.set(id, new GeoPoint(timestamp, latitude, longitude));
		this.wait(waitTime);
	}
}
